<iframe src="../.ibox.html?raw=true" style="border:none; position:fixed; width:40px; right:0; z-index=999;"></iframe>

# Number Formatting

Number formatting is primarily used in tables to format a column or row of numbers with a more
presentable representation. Number formatting has a specific syntax which follows the form:
```
prefix[[.#|~#|.*][%|s|b|n][/<char>]]suffix
^----^ ^--------^^-------^^-------^ ^----^
prefix precision scale    separator suffix
```

## Prefix and Suffix
The prefix and suffix are bound to the supported lengths, and can have any character in them.
To use `[]` characters, a double bracket must be used.

### Example
> Number being used is `1.234`
```kserd
render-as = "table"
header = [ "format", "example" ]
data = [
    ["<blank>", 1.234]
    ["prefix ", 1.234]
    ["[] suffix", 1.234]
    ["[[prefix [] suffix]]", 1.234]
]
[numfmts]
    rows = {
	1: ""
	2: "prefix "
	3: "[] suffix"
	4: "[[prefix [] suffix]]"
    }
```

## Precision
Precision is defined using a `.` for decimals, or a `~` for significant figures, followed by
a number. A maximum number of 255 is supported. There is a special case: `.*`, which removes any
default precision and uses an _unspecified_ amount of precision.

### Example
> Number being used is `1.234`
```kserd
render-as = "table"
header = [ "format", "example" ]
data = [
    ["[.2]", 1.234]
    ["[.0]", 1.234]
    ["[.*]", 1.234]
    ["[~3]", 1.234]
]
[numfmts]
    rows = {
	1: "[.2]"
	2: "[.0]"
	3: "[.*]"
	4: "[~3]"
    }
```

## Scale
Scale uses a character to denote what scaling should be used. By default the SI scaling is
used. The following characters are supported:
- `s` for SI scaling
- `%` for percentage scaling
- `m` for metric scaling
- `b` for binary scaling
- `n` for no scaling

### Example
> Number being used is `1,234,567,890`
```kserd
render-as = "table"
header = [ "format", "example" ]
data = [
    ["<default>", 1234567890]
    ["[n]", 1234567890]
    ["[%]", 1234567890]
    ["[m]", 1234567890]
    ["[b]", 1234567890]
]
[numfmts]
    rows = {
	1: ""
	2: "[n]"
	3: "[%]"
	4: "[m]"
	5: "[b]"
    }
```

## Separator
A separator character can be specified by using a forward slash `/` followed by a character.
The parser uses the _next character verbatim_, unless that character is `]` in which case the
separator is set to `None`. The default separator is a comma.

### Example
> Number being used is `1,234,567,890`
```kserd
render-as = "table"
header = [ "format", "example" ]
data = [
    ["<default>", 1234567890]
    ["[n/]", 1234567890]
    ["[.0n/.]", 1234567890]
    ["[n/_]", 1234567890]
    ["[n/ ]", 1234567890]
]
[numfmts]
    rows = {
	1: "[n]"
	2: "[n/]"
	3: "[.0n/.]"
	4: "[n/_]"
	5: "[n/ ]"
    }
```

## Composing formats
There have been examples of composing formats already. The `prefix[num]suffix` order must be
adhered to, but the ordering within the number format is arbitrary. It is recommended to keep it
consistent with _precison, scaling, separator_ as this assists with readability and lowers the
risk of malformed formats (which will error on the parsing phase).

### Various composed examples
```kserd
render-as = "table"
header = [ "desc", "number", "format", "example" ]
data = [
    ["Percentages to two decimal places", "0.012345", "[.2%]", 0.012345]
    ["Currency to zero decimal places", "123456789.12345", "$[.0n] USD", 123456789.12345]
    ["Formatting file size", "123456789", "[~3b]B", 123456789]
    ["Units to 1 decimal place", "12345.68", "[.1n] m/s", 12345.68]
]
[numfmts]
    rows = {
	1: "[.2%]"
	2: "$[.0n] USD"
	3: "[~3b]B"
	4: "[.1n] m/s"
    }
```
